@@ -2,9 +2,9 @@
 
 Implements the Distutils 'sdist' command (create a source distribution)."""
 
-# This module should be kept compatible with Python 1.5.2.
+# This module should be kept compatible with Python 2.1.
 
-__revision__ = "$Id: sdist.py 37012 2004-08-16 12:15:00Z doko $"
+__revision__ = "$Id: sdist.py 37828 2004-11-10 22:23:15Z loewis $"
 
 import sys, os, string
 from types import *
@@ -347,7 +347,7 @@ class sdist (Command):
           * the build tree (typically "build")
           * the release tree itself (only an issue if we ran "sdist"
             previously with --keep-temp, or it aborted)
-          * any RCS or CVS directories
+          * any RCS, CVS and .svn directories
         """
         build = self.get_finalized_command('build')
         base_dir = self.distribution.get_fullname()
